import libcst as cst
from libcst.codemod import ContextAwareTransformer

from u2p.transformers import (
    ReplaceAssertEqualTransfomer,
    ReplaceAssertRaisesTransformer,
    ReplaceAssertTrueTransformer,
)
from u2p.transformers.generic import RemoveBaseClassTransformer


class U2PTransformer(ContextAwareTransformer):
    def transform_module_impl(self, tree: cst.Module) -> cst.Module:
        transformers: list[ContextAwareTransformer] = [
            ReplaceAssertEqualTransfomer(self.context),
            ReplaceAssertRaisesTransformer(self.context),
            ReplaceAssertTrueTransformer(self.context),
            RemoveBaseClassTransformer(self.context, "TestCase"),
            RemoveBaseClassTransformer(self.context, "unittest.TestCase"),
        ]

        for transformer in transformers:
            tree = transformer.transform_module(tree)

        return tree



from .bases import RemoveBaseClassTransformer
from .imports import AddImportTransformer, RemoveImportTransformer

__all__ = (
    "AddImportTransformer",
    "RemoveBaseClassTransformer",
    "RemoveImportTransformer",
)



from typing import Optional

import libcst as cst
from libcst.codemod import CodemodContext, ContextAwareTransformer
from libcst.codemod.visitors import AddImportsVisitor, RemoveImportsVisitor

__all__ = (
    "AddImportTransformer",
    "RemoveImportTransformer",
)


class AddImportTransformer(ContextAwareTransformer):
    """
    A wrapper for AddImportsVisitor from libcst providing
    an interface consistent with all other transformers.
    """

    def __init__(
        self,
        context: CodemodContext,
        module: str,
        obj: Optional[str] = None,
        asname: Optional[str] = None,
        relative: int = 0,
    ) -> None:
        super().__init__(context)
        self._args = (module, obj, asname, relative)

    def transform_module_impl(self, tree: cst.Module) -> cst.Module:
        AddImportsVisitor.add_needed_import(self.context, *self._args)
        return AddImportsVisitor(self.context).transform_module(tree)


class RemoveImportTransformer(ContextAwareTransformer):
    """
    A wrapper for RemoveImportsVisitor from libcst providing
    an interface consistent with all other transformers.
    """

    def __init__(
        self,
        context: CodemodContext,
        module: str,
        obj: Optional[str] = None,
        asname: Optional[str] = None,
    ) -> None:
        super().__init__(context)
        self._args = (module, obj, asname)

    def transform_module_impl(self, tree: cst.Module) -> cst.Module:
        RemoveImportsVisitor.remove_unused_import(self.context, *self._args)
        return RemoveImportsVisitor(self.context).transform_module(tree)



import argparse

import libcst as cst
from libcst.codemod import CodemodContext

from u2p.transformers import U2PTransformer


def _run_transformer(source: str) -> str:
    tree = cst.parse_module(source)
    transformer = U2PTransformer(CodemodContext())
    tree = transformer.transform_module(tree)
    return tree.code


def run(argv=None):
    parser = argparse.ArgumentParser("u2p")
    parser.add_argument("file")
    parser.add_argument(
        "-w",
        "--write",
        action="store_true",
        help="Write back modified files.",
    )

    try:
        args = parser.parse_args(argv)
    except SystemExit as exc:
        return exc.code

    with open(args.file) as f:
        out = _run_transformer(f.read())

    if args.write:
        with open(args.file, "w") as f:
            f.write(out)
    else:
        print(out)

    return 0



import libcst as cst
import libcst.matchers as m
from libcst.codemod import CodemodContext, ContextAwareTransformer


class RemoveBaseClassTransformer(ContextAwareTransformer):
    def __init__(self, context: CodemodContext, base: str) -> None:
        super().__init__(context)

        if len(parts := base.split(".", maxsplit=1)) == 2:
            value, attr = parts
            self._base = m.Attribute(
                value=m.Name(value=value),
                attr=m.Name(value=attr),
            )
        else:
            self._base = m.Name(value=base)

    def leave_ClassDef(self, original_node, updated_node):
        updated_bases = [
            base for base in updated_node.bases if not m.matches(base.value, self._base)
        ]
        updated = len(updated_bases) != len(updated_node.bases)

        if updated:
            updated_node = updated_node.with_changes(bases=updated_bases)

            # If no more bases, remove parentheses.
            if len(updated_node.bases) == 0:
                updated_node = updated_node.with_changes(
                    lpar=cst.MaybeSentinel.DEFAULT,
                    rpar=cst.MaybeSentinel.DEFAULT,
                )

        return updated_node



import libcst as cst
from libcst.codemod import ContextAwareTransformer


def transform(transformer: ContextAwareTransformer, source: str) -> str:
    tree = cst.parse_module(source)
    tree = transformer.transform_module(tree)
    return tree.code



import libcst as cst
import libcst.matchers as m
from libcst.codemod import CodemodContext, ContextAwareTransformer

__all__ = (
    "ReplaceAssertEqualTransfomer",
    "ReplaceAssertTrueTransformer",
    "ReplaceAssertRaisesTransformer",
)

AssertTrue = m.Attribute(
    value=m.Name(value="self"),
    attr=m.Name(value="assertTrue"),
)
AssertEqual = m.Attribute(
    value=m.Name(value="self"),
    attr=m.Name(value="assertEqual"),
)
AssertRaises = m.Attribute(
    value=m.Name(value="self"),
    attr=m.Name(value="assertRaises"),
)


class ReplaceAssertTrueTransformer(ContextAwareTransformer):
    def __init__(self, context: CodemodContext) -> None:
        super().__init__(context)

    def leave_Call(self, original_node: cst.Call, updated_node: cst.Call) -> cst.Call:
        if m.matches(updated_node.func, AssertTrue):
            return cst.Assert(updated_node.args[0].value)
        return updated_node


class ReplaceAssertEqualTransfomer(ContextAwareTransformer):
    def __init__(self, context: CodemodContext) -> None:
        super().__init__(context)

    def leave_Call(self, original_node: cst.Call, updated_node: cst.Call) -> cst.Call:
        if m.matches(updated_node.func, AssertEqual):
            return cst.Assert(
                test=cst.Comparison(
                    left=updated_node.args[0].value,
                    comparisons=[
                        cst.ComparisonTarget(
                            operator=cst.Equal(),
                            comparator=updated_node.args[1].value,
                        )
                    ],
                )
            )
        return updated_node


class ReplaceAssertRaisesTransformer(ContextAwareTransformer):
    def __init__(self, context: CodemodContext) -> None:
        super().__init__(context)

    def leave_Call(self, original_node: cst.Call, updated_node: cst.Call) -> cst.Call:
        if m.matches(updated_node.func, AssertRaises):
            return updated_node.with_changes(
                func=cst.Attribute(
                    value=cst.Name(
                        value="pytest",
                    ),
                    attr=cst.Name(
                        value="raises",
                    ),
                ),
            )
        return updated_node



from .asserts import (
    ReplaceAssertEqualTransfomer,
    ReplaceAssertRaisesTransformer,
    ReplaceAssertTrueTransformer,
)
from .main import U2PTransformer

__all__ = (
    "ReplaceAssertEqualTransfomer",
    "ReplaceAssertRaisesTransformer",
    "ReplaceAssertTrueTransformer",
    "U2PTransformer",
)
